@@ -1,6 +1,7 @@
 #include "ap_config.h"
 #include "ap_mmn.h"
 #include "apr_strings.h"
+#include "apr_hash.h"
 #include "http_main.h"
 #include "httpd.h"
 #include "http_config.h"
@@ -16,6 +17,7 @@ extern module AP_MODULE_DECLARE_DATA fcgid_module;
 #define DEFAULT_ZOMBIE_SCAN_INTERVAL 3
 #define DEFAULT_PROC_LIFETIME (60*60)
 #define DEFAULT_SOCKET_PREFIX "logs/fcgidsock"
+#define DEFAULT_SHM_PATH "logs/fcgid_shm"
 #define DEFAULT_SPAWNSOCRE_UPLIMIT 10
 #define DEFAULT_SPAWN_SCORE 1
 #define DEFAULT_TERMINATION_SCORE 2
@@ -23,7 +25,7 @@ extern module AP_MODULE_DECLARE_DATA fcgid_module;
 #define DEFAULT_MAX_CLASS_PROCESS_COUNT 100
 #define DEFAULT_MIN_CLASS_PROCESS_COUNT 3
 #define DEFAULT_IPC_CONNECT_TIMEOUT 3
-#define DEFAULT_IPC_COMM_TIMEOUT 20
+#define DEFAULT_IPC_COMM_TIMEOUT 40
 #define DEFAULT_OUTPUT_BUFFERSIZE 65536
 #define DEFAULT_MAX_REQUESTS_PER_PROCESS -1
 
@@ -34,6 +36,7 @@ void *create_fcgid_server_config(apr_pool_t * p, server_rec * s)
 	config->default_init_env = apr_table_make(p, 20);
 	config->sockname_prefix =
 		ap_server_root_relative(p, DEFAULT_SOCKET_PREFIX);
+	config->shmname_path = ap_server_root_relative(p, DEFAULT_SHM_PATH);
 	config->idle_timeout = DEFAULT_IDLE_TIMEOUT;
 	config->idle_scan_interval = DEFAULT_IDLE_SCAN_INTERVAL;
 	config->busy_scan_interval = DEFAULT_BUSY_SCAN_INTERVAL;
@@ -245,6 +248,25 @@ const char *get_socketpath(server_rec * s)
 	return config->sockname_prefix;
 }
 
+const char *set_shmpath(cmd_parms * cmd, void *dummy, const char *arg)
+{
+	server_rec *s = cmd->server;
+	fcgid_server_conf *config =
+		ap_get_module_config(s->module_config, &fcgid_module);
+	config->shmname_path = ap_server_root_relative(cmd->pool, arg);
+	if (!config->shmname_path)
+		return "Invalid shmname path";
+
+	return NULL;
+}
+
+const char *get_shmpath(server_rec * s)
+{
+	fcgid_server_conf *config =
+		ap_get_module_config(s->module_config, &fcgid_module);
+	return config->shmname_path;
+}
+
 const char *set_spawnscore_uplimit(cmd_parms * cmd, void *dummy,
 								   const char *arg)
 {
@@ -494,8 +516,7 @@ const char *set_authenticator_authoritative(cmd_parms * cmd,
 	return NULL;
 }
 
-fcgid_wrapper_conf *get_authenticator_info(request_rec * r,
-										   int *authoritative)
+auth_conf *get_authenticator_info(request_rec * r, int *authoritative)
 {
 	fcgid_dir_conf *config =
 		ap_get_module_config(r->per_dir_config, &fcgid_module);
@@ -547,8 +568,7 @@ const char *set_authorizer_authoritative(cmd_parms * cmd,
 	return NULL;
 }
 
-fcgid_wrapper_conf *get_authorizer_info(request_rec * r,
-										int *authoritative)
+auth_conf *get_authorizer_info(request_rec * r, int *authoritative)
 {
 	fcgid_dir_conf *config =
 		ap_get_module_config(r->per_dir_config, &fcgid_module);
@@ -599,7 +619,7 @@ const char *set_access_authoritative(cmd_parms * cmd,
 	return NULL;
 }
 
-fcgid_wrapper_conf *get_access_info(request_rec * r, int *authoritative)
+auth_conf *get_access_info(request_rec * r, int *authoritative)
 {
 	fcgid_dir_conf *config =
 		ap_get_module_config(r->per_dir_config, &fcgid_module);
@@ -612,12 +632,21 @@ fcgid_wrapper_conf *get_access_info(request_rec * r, int *authoritative)
 	return NULL;
 }
 
+typedef struct {
+	apr_hash_t *wrapper_id_hash;
+	apr_size_t cur_id;
+} wrapper_id_info;
+
 const char *set_wrapper_config(cmd_parms * cmd, void *dirconfig,
 							   const char *wrapperpath,
 							   const char *extension)
 {
+	const char *path, *tmp;
 	apr_status_t rv;
 	apr_finfo_t finfo;
+	const char *userdata_key = "fcgid_wrapper_id";
+	wrapper_id_info *id_info;
+	apr_size_t *wrapper_id;
 	fcgid_wrapper_conf *wrapper = NULL;
 	fcgid_dir_conf *config = (fcgid_dir_conf *) dirconfig;
 
@@ -627,23 +656,52 @@ const char *set_wrapper_config(cmd_parms * cmd, void *dirconfig,
 		|| strchr(extension, '/') || strchr(extension, '\\'))
 		return "Invalid wrapper file extension";
 
-	/* Is the wrapper exist? */
-	if ((rv = apr_stat(&finfo, wrapperpath, APR_FINFO_NORM,
-					   cmd->temp_pool)) != APR_SUCCESS) {
-		return apr_psprintf(cmd->pool,
-							"can't get fastcgi file info: %s, errno: %d",
-							wrapperpath, apr_get_os_error());
+	/* Get wrapper_id base on wrapperpath */
+	apr_pool_userdata_get((void *) &id_info, userdata_key,
+						  cmd->server->process->pool);
+	if (!id_info) {
+		id_info =
+			apr_pcalloc(cmd->server->process->pool, sizeof(*id_info));
+		id_info->wrapper_id_hash =
+			apr_hash_make(cmd->server->process->pool);
+		apr_pool_userdata_set((const void *) id_info, userdata_key,
+							  apr_pool_cleanup_null,
+							  cmd->server->process->pool);
+	}
+	if ((wrapper_id =
+		 apr_hash_get(id_info->wrapper_id_hash, wrapperpath,
+					  strlen(wrapperpath))) == NULL) {
+		wrapper_id =
+			apr_pcalloc(cmd->server->process->pool, sizeof(*wrapper_id));
+		*wrapper_id = id_info->cur_id++;
+		apr_hash_set(id_info->wrapper_id_hash, wrapperpath,
+					 strlen(wrapperpath), wrapper_id);
 	}
 
-	/* Create the wrapper node */
 	wrapper = apr_pcalloc(cmd->server->process->pconf, sizeof(*wrapper));
 	if (!wrapper)
 		return "Can't alloc memory for wrapper";
-	strncpy(wrapper->path, wrapperpath, _POSIX_PATH_MAX - 1);
-	wrapper->path[_POSIX_PATH_MAX - 1] = '\0';
+
+	/* Get wrapper path */
+	tmp = wrapperpath;
+	path = ap_getword_white(cmd->temp_pool, &tmp);
+	if (path == NULL || *path == '\0')
+		return "Invalid wrapper config";
+
+	/* Is the wrapper exist? */
+	if ((rv = apr_stat(&finfo, path, APR_FINFO_NORM,
+					   cmd->temp_pool)) != APR_SUCCESS) {
+		return apr_psprintf(cmd->pool,
+							"can't get fastcgi file info: %s(%s), errno: %d",
+							wrapperpath, path, apr_get_os_error());
+	}
+
+	strncpy(wrapper->args, wrapperpath, _POSIX_PATH_MAX - 1);
+	wrapper->args[_POSIX_PATH_MAX - 1] = '\0';
 	wrapper->inode = finfo.inode;
 	wrapper->deviceid = finfo.device;
-	wrapper->share_group_id = (apr_size_t) - 1;
+	wrapper->share_group_id = *wrapper_id;
+	*wrapper_id++;
 
 	/* Add the node now */
 	apr_hash_set(config->wrapper_info_hash, extension, strlen(extension),
