@@ -201,7 +201,8 @@ static int mod_fcgid_authenticator(request_rec * r)
 	const char *password = NULL;
 	const char *location = NULL;
 	apr_table_t *saved_subprocess_env = NULL;
-	fcgid_wrapper_conf *wrapper_conf, *authenticator_info;
+	fcgid_wrapper_conf *wrapper_conf;
+	auth_conf *authenticator_info;
 	int authoritative;
 
 	authenticator_info = get_authenticator_info(r, &authoritative);
@@ -270,7 +271,7 @@ static int mod_fcgid_authenticator(request_rec * r)
 						  r->user);
 
 		/* Handle error */
-		if (authoritative)
+		if (!authoritative)
 			return DECLINED;
 		else {
 			ap_note_basic_auth_failure(r);
@@ -284,7 +285,8 @@ static int mod_fcgid_authorizer(request_rec * r)
 	int res = 0;
 	const char *location = NULL;
 	apr_table_t *saved_subprocess_env = NULL;
-	fcgid_wrapper_conf *wrapper_conf, *authorizer_info;
+	fcgid_wrapper_conf *wrapper_conf;
+	auth_conf *authorizer_info;
 	int authoritative;
 
 	authorizer_info = get_authorizer_info(r, &authoritative);
@@ -348,7 +350,7 @@ static int mod_fcgid_authorizer(request_rec * r)
 						  r->user);
 
 		/* Handle error */
-		if (authoritative)
+		if (!authoritative)
 			return DECLINED;
 		else {
 			ap_note_basic_auth_failure(r);
@@ -362,7 +364,8 @@ static int mod_fcgid_check_access(request_rec * r)
 	int res = 0;
 	const char *location = NULL;
 	apr_table_t *saved_subprocess_env = NULL;
-	fcgid_wrapper_conf *wrapper_conf, *access_info;
+	fcgid_wrapper_conf *wrapper_conf;
+	auth_conf *access_info;
 	int authoritative;
 
 	access_info = get_access_info(r, &authoritative);
@@ -427,7 +430,7 @@ static int mod_fcgid_check_access(request_rec * r)
 						  r->user);
 
 		/* Handle error */
-		if (authoritative)
+		if (!authoritative)
 			return DECLINED;
 		else {
 			ap_note_basic_auth_failure(r);
@@ -530,6 +533,8 @@ static const command_rec fcgid_cmds[] = {
 				  "fastcgi application lifetime"),
 	AP_INIT_TAKE1("SocketPath", set_socketpath, NULL, RSRC_CONF,
 				  "fastcgi socket file path"),
+	AP_INIT_TAKE1("SharememPath", set_shmpath, NULL, RSRC_CONF,
+				  "fastcgi share memory file path"),
 	AP_INIT_TAKE1("SpawnScoreUpLimit", set_spawnscore_uplimit, NULL,
 				  RSRC_CONF,
 				  "Spawn score up limit"),
@@ -558,7 +563,8 @@ static const command_rec fcgid_cmds[] = {
 				  "Communication timeout to fastcgi server"),
 	AP_INIT_TAKE12("DefaultInitEnv", add_default_env_vars, NULL, RSRC_CONF,
 				   "an environment variable name and optional value to pass to FastCGI."),
-	AP_INIT_TAKE12("FCGIWrapper", set_wrapper_config, NULL, ACCESS_CONF,
+	AP_INIT_TAKE12("FCGIWrapper", set_wrapper_config, NULL,
+				   RSRC_CONF | ACCESS_CONF | OR_FILEINFO,
 				   "The CGI wrapper setting"),
 	AP_INIT_TAKE1("PHP_Fix_Pathinfo_Enable",
 				  set_php_fix_pathinfo_enable,
@@ -568,22 +574,24 @@ static const command_rec fcgid_cmds[] = {
 				  NULL, RSRC_CONF,
 				  "Max requests handled by each fastcgi application"),
 	AP_INIT_TAKE1("FastCgiAuthenticator", set_authenticator_info, NULL,
-				  ACCESS_CONF,
+				  ACCESS_CONF | OR_FILEINFO,
 				  "a absolute authenticator file path"),
 	AP_INIT_FLAG("FastCgiAuthenticatorAuthoritative",
-				 set_authenticator_authoritative, NULL, ACCESS_CONF,
+				 set_authenticator_authoritative, NULL,
+				 ACCESS_CONF | OR_FILEINFO,
 				 "Set to 'off' to allow authentication to be passed along to lower modules upon failure"),
 	AP_INIT_TAKE1("FastCgiAuthenticator", set_authorizer_info, NULL,
-				  ACCESS_CONF,
+				  ACCESS_CONF | OR_FILEINFO,
 				  "a absolute authorizer file path"),
 	AP_INIT_FLAG("FastCgiAuthenticatorAuthoritative",
-				 set_authorizer_authoritative, NULL, ACCESS_CONF,
+				 set_authorizer_authoritative, NULL,
+				 ACCESS_CONF | OR_FILEINFO,
 				 "Set to 'off' to allow authorization to be passed along to lower modules upon failure"),
 	AP_INIT_TAKE1("FastCgiAccessChecker", set_access_info, NULL,
-				  ACCESS_CONF,
+				  ACCESS_CONF | OR_FILEINFO,
 				  "a absolute access checker file path"),
 	AP_INIT_FLAG("FastCgiAccessCheckerAuthoritative",
-				 set_access_authoritative, NULL, ACCESS_CONF,
+				 set_access_authoritative, NULL, ACCESS_CONF | OR_FILEINFO,
 				 "Set to 'off' to allow access control to be passed along to lower modules upon failure"),
 	{NULL}
 };
